@@ -174,7 +174,7 @@ def dedupe_yarn(path, logger=None):
         pins above, for example, known-bad versions
     """
     had_dupes = ProgressProcess(
-        ['node', YARN_PATH, 'yarn-deduplicate', '-s', 'fewer'],
+        ['node', YARN_PATH, 'yarn-deduplicate', '-s', 'fewer', '--fail'],
         cwd=path, logger=logger
     ).wait() != 0
 
@@ -1202,6 +1202,7 @@ class _AppHandler(object):
         # Handle local extensions.
         for (key, source) in local.items():
             jlab['linkedPackages'][key] = source
+            data['resolutions'][key] = source
 
         # Handle linked packages.
         for (key, item) in linked.items():
@@ -1209,6 +1210,7 @@ class _AppHandler(object):
             path = pjoin(path, item['filename'])
             data['dependencies'][key] = format_path(path)
             jlab['linkedPackages'][key] = item['source']
+            data['resolutions'][key] = format_path(path)
 
         # Handle extensions
         compat_errors = self._get_extension_compat()
@@ -1237,10 +1239,11 @@ class _AppHandler(object):
         for item in self.info['uninstalled_core']:
             if item in jlab['extensions']:
                 data['jupyterlab']['extensions'].pop(item)
-            else:
+            elif item in jlab['mimeExtensions']:
                 data['jupyterlab']['mimeExtensions'].pop(item)
             # Remove from dependencies as well.
-            data['dependencies'].pop(item)
+            if item in data['dependencies']:
+                data['dependencies'].pop(item)
 
         return data
 
@@ -1264,6 +1267,9 @@ class _AppHandler(object):
         """
         # Extract the package in a temporary directory.
         existing = data['filename']
+        if not osp.exists(pjoin(dname, existing)):
+            existing = ''
+
         with TemporaryDirectory() as tempdir:
             info = self._extract_package(source, tempdir)
 
@@ -1273,7 +1279,7 @@ class _AppHandler(object):
 
             shutil.move(info['path'], pjoin(dname, info['filename']))
 
-        # Remove the existing tarball and return the new file name.
+        # Remove the previous tarball and return the new file name.
         if existing:
             os.remove(pjoin(dname, existing))
 
@@ -1501,11 +1507,25 @@ class _AppHandler(object):
         info = self._extract_package(extension, tempdir, pin=pin)
         data = info['data']
 
+        # Check for compatible version unless:
+        # - A specific version was requested (@ in name,
+        #   but after first char to allow for scope marker).
+        # - Package is locally installed.
+        allow_fallback = '@' not in extension[1:] and not info['is_dir']
+        name = info['name']
+
         # Verify that the package is an extension.
         messages = _validate_extension(data)
         if messages:
             msg = '"%s" is not a valid extension:\n%s'
-            raise ValueError(msg % (extension, '\n'.join(messages)))
+            msg = msg % (extension, '\n'.join(messages))
+            if allow_fallback:
+                try:
+                    version = self._latest_compatible_package_version(name)
+                except URLError:
+                    raise ValueError(msg)
+            else:
+                raise ValueError(msg)
 
         # Verify package compatibility.
         deps = data.get('dependencies', dict())
@@ -1514,12 +1534,7 @@ class _AppHandler(object):
             msg = _format_compatibility_errors(
                 data['name'], data['version'], errors
             )
-            # Check for compatible version unless:
-            # - A specific version was requested (@ in name,
-            #   but after first char to allow for scope marker).
-            # - Package is locally installed.
-            if '@' not in extension[1:] and not info['is_dir']:
-                name = info['name']
+            if allow_fallback:
                 try:
                     version = self._latest_compatible_package_version(name)
                 except URLError:
